/*
 * packet.h
 *
 * Header file for the packet routines.
 *
 * Kekoa Proudfoot 12/30/96
 */

#ifndef PACKET_H_DEFINED
#define PACKET_H_DEFINED

#include <stdio.h>

/*****************************************************************************
 * Configuration macros
 *****************************************************************************/

#define PACKET_MAX_DATA_SIZE 8192

/*****************************************************************************
 * Type definitions
 *****************************************************************************/

/* Enum of packet types */

typedef enum packet_type_t {

    /* Bad packet types */
    
    PACKET_UNDEFINED       = 0x0000,

    /* Primary packet types */

    PACKET_CONTROL         = 0x8000,
    PACKET_RELIABLE        = 0x0004,
    PACKET_UNRELIABLE      = 0x0010,

    /* Secondary (internal) packet types */

    PACKET_RELIABLE_REPLY  = 0x0002,
    PACKET_RELIABLE_MORE   = 0x0001,
    PACKET_RELIABLE_END    = 0x0009

} packet_type_t;

typedef struct packet_header_t {
    unsigned short  type;
    unsigned short  length;
    unsigned long   number; /* beware, not all packets have this */
} packet_header_t;

/* Primary packet structs */

typedef struct packet_control_t {
    unsigned short  type;
    unsigned short  length;
    char            data[PACKET_MAX_DATA_SIZE];
} packet_control_t;

typedef struct packet_reliable_t {
    unsigned short  type;
    unsigned short  length;
    unsigned long   number;
    char            data[PACKET_MAX_DATA_SIZE];
} packet_reliable_t;

typedef struct packet_unreliable_t {
    unsigned short  type;
    unsigned short  length;
    unsigned long   number;
    char            data[PACKET_MAX_DATA_SIZE];
} packet_unreliable_t;

/* Union of all packet structs */

typedef union packet_t {
    unsigned short            type;

    packet_header_t           header;

    packet_control_t          control;
    packet_reliable_t         reliable;
    packet_unreliable_t       unreliable;
} packet_t;

/*****************************************************************************
 * Interface function declarations
 *****************************************************************************/

extern packet_t *packet_create          (unsigned short type);
extern void      packet_destroy         (packet_t *packet);
extern void      packet_copy            (packet_t *dst, packet_t *src);
extern char     *packet_name            (packet_t *packet);
extern void      packet_cl_to_sv_debug  (packet_t *packet, FILE *file);
extern void      packet_sv_to_cl_debug  (packet_t *packet, FILE *file);
extern void      packet_host_to_network (packet_t *packet);
extern void      packet_network_to_host (packet_t *packet);

#endif
